import Flutter
import UIKit
import raveniosdspreadpos

public class MobilePosPlugin:  FlutterAppDelegate,FlutterPlugin,TransactionNotify, delegateError,RavenMobilePosCallBack {

    
      let manager = Manager()
      var baseUrl: String="";
      var result: FlutterResult?
      var availableDevices = [BluetoothModel]()
    
       static var methodChannel: FlutterMethodChannel?
      var  ravenMPos: RavenMobilePos?;
    
      public override init() {
          super.init()
          manager.delegate=self;
      }
      static public func register(with registrar: FlutterPluginRegistrar) {
          
        let channel = FlutterMethodChannel(name: "mobile_pos", binaryMessenger: registrar.messenger())
        let instance = MobilePosPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
          methodChannel=channel;
          
  }
    

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
      self.result=result;
      if(call.method=="setBaseUrl"){
          let args = call.arguments as! Dictionary<String, Any>;
          baseUrl=(args["base_url"] as? String)!;
          if(baseUrl.hasSuffix("/")){
              baseUrl.removeLast()
          }
          result("success")
          return;
      }
      
      if(call.method=="startTransaction"){
          let args = call.arguments as! Dictionary<String, Any>;
          startTransaction(result: result,args: args);
          return;
      }
      
      if(call.method=="getPosSdkStatus"){
          result(true);
          return;
      }
      if(call.method==RavenMPosConst.disconnectDevice){
          self.ravenMPos?.disconnectDevice()
          result(true);
          return;
      }
      if(call.method==RavenMPosConst.scanForMPosDevice){
          self.ravenMPos?.refreshScan();
          result(true)
      }
      if(call.method==RavenMPosConst.onRequestBluetoothConnection){
          let args = call.arguments as! Dictionary<String, Any>;
          processBluetoothConnection(address: args["address"] as! String)
          return;
      }
      if(call.method==RavenMPosConst.onPinProvided){
          let args = call.arguments as! Dictionary<String, Any>;
          processUserPin(pin: args["pin"]as?String ?? "")
          return;
      }
      result("nothing implemented");
  }
    public func startTransaction(result: FlutterResult,args:Dictionary<String, Any>){
        let rd = RequestData()
        rd.AccType=args["account_type"] as? String;
        rd.Transaction_Amount = ("\(args["amount"]!)");
        rd.Terminal_ID = args["terminal_id"] as? String;
        rd.BluetoothType = true;
        rd.ClearMasterKey = args["master_key"] as? String;
        rd.ClearPinKey = args["pin_key"] as? String;
        rd.Port = args["port"] as? String;
        rd.IP  = args["ip"] as? String;
        rd.MID = args["mid"] as? String;
        rd.SN = args["serial_number"] as? String;
        rd.BusinessName = args["business_name"] as? String;
        rd.ClearSessionKey = args["session_key"] as? String;
        rd.BaseURL = baseUrl
        ravenMPos = self.manager.startRavenMPOS(request: rd, selfController: self,transactionNotify: self, callbacks: self)
        self.result!("PAYMENT INITIATED SUCCESSFULLY");
        
    }
    
    public func processUserPin(pin:String){
        ravenMPos?.processUserPin(pin: pin)
        self.result!("I GOT THE PIN THANKS___________**________")
    }
    
    public func processBluetoothConnection(address:String){
        
        if(self.availableDevices.isEmpty){
            var args:Dictionary<String, Any> = ["error":"device no found... please scan and try again."]
            MobilePosPlugin.methodChannel?.invokeMethod(RavenMPosConst.onError, arguments: args)
            return;
        }
        
        
        for device:BluetoothModel in self.availableDevices{
            if(device.address==address){
                ravenMPos?.processBluetoothConnection(selectedDevice: device)
                self.result!("******________********DEVICE CONNECTING NOW___________**________")
                break;
            }
        }
       
    }
    
    
    public func alertCallback(message: String) {
        print("***********************");
        print("ALERT CALLBACK TRIGGERED");
        print("***********************");
        let args:Dictionary<String, Any> = ["alert":message]
        print(args)
        MobilePosPlugin.methodChannel?.invokeMethod(RavenMPosConst.onAlert,arguments: args)
    }
    
    public func onErrorCallback(errorMsg: String) {
        print("***********************");
        print("ERROR CALLBACK TRIGGERED");
        print("***********************");
        let args:Dictionary<String, Any> = ["error":errorMsg]
        MobilePosPlugin.methodChannel?.invokeMethod(RavenMPosConst.onError,arguments: args)
    }
    
    public func onBluetoothDeviceDetected(_ availableDevices: raveniosdspreadpos.BluetoothObjectWrapper) {
        print("***********************");
        print("BLUETOOTH DEVICE DETECTED  CALLBACK TRIGGERED");
        print("***********************");
        self.availableDevices=availableDevices.object.obj 
        let encodedDevice = try! JSONEncoder().encode(self.availableDevices)
        let stringData = String(decoding: encodedDevice, as: UTF8.self);        MobilePosPlugin.methodChannel?.invokeMethod(RavenMPosConst.onBluetoothDeviceDetected,arguments: stringData)
    }
    
    public func onBlutoothConnected() {
        print("***********************");
        print("BLUETOOTH CONNECTED SUCCESSFULLY CALLBACK TRIGGERED");
        print("***********************");
        MobilePosPlugin.methodChannel?.invokeMethod(RavenMPosConst.onBluetoothConnected,arguments: nil)
    }
    
    public func onBlutoothConnectionError() {
        print("***********************");
        print("BLUETOOTH CONNECTION ERROR CALLBACK TRIGGERED");
        print("***********************");
        MobilePosPlugin.methodChannel?.invokeMethod(RavenMPosConst.onBluetoothConnectionError,arguments: nil)
    }
    public func onBluetoothDisconnected() {
        print("***********************");
        print("BLUETOOTH DISCONNECTED CALLBACK TRIGGERED");
        print("***********************");
        MobilePosPlugin.methodChannel?.invokeMethod(RavenMPosConst.onBluetoothDisconnected,arguments: nil)
    }
    
    public func bluethootStartScaning() {
        print("***********************");
        print("BLUETOOTH START SCANNING CALLBACK TRIGGERED");
        print("***********************");
        MobilePosPlugin.methodChannel?.invokeMethod(RavenMPosConst.onBluetoothStartScanning,arguments: nil)
    }
    
    public func bluethootStopScaning() {
        print("***********************");
        print("BLUETOOTH STOP SCANNING CALLBACK TRIGGERED");
        print("***********************");
        DispatchQueue.main.async {
            MobilePosPlugin.methodChannel?.invokeMethod(RavenMPosConst.onBluetoothStopScanning,arguments: nil)
        }
    }
    
    public func onTransactionError(message: String) {
        print("***********************");
        print("TRANSACTION ERROR CALLBACK TRIGGERED");
        print("***********************");
        let args:Dictionary<String, Any> = ["error":message]
        MobilePosPlugin.methodChannel?.invokeMethod(RavenMPosConst.onTransactionError,arguments: args)
    }
    
    
    public func onRequestPin() {
        print("***********************");
        print("REQUEST PIN CALLBACK TRIGGERED");
        print("***********************");
        MobilePosPlugin.methodChannel?.invokeMethod(RavenMPosConst.onRequestPin,arguments: nil)
    }
    
    public func onRequestInsertCard() {
        print("***********************");
        print("REQUEST CARD CALLBACK TRIGGERED");
        print("***********************");
        MobilePosPlugin.methodChannel?.invokeMethod(RavenMPosConst.onRequestCard,arguments: nil)
    }
    
    public func ErrorMessage(errormsg: String?) {
        print("***********************");
        print("ERROR CALLBACK TRIGGERED");
        print("***********************");
        MobilePosPlugin.methodChannel?.invokeMethod(RavenMPosConst.onError,arguments: nil)
    }
    
    public func RavenEmv(responseModel: raveniosdspreadpos.RavenEmv?) {
        let mraw = try! JSONEncoder().encode(responseModel)
        let stringData = String(decoding: mraw, as: UTF8.self)
        print("***********************");
        print("TRANSACTION PAYLOAD GENERATED SUCCESSFULLY");
        print("***********************");
        MobilePosPlugin.methodChannel?.invokeMethod(RavenMPosConst.onProcessCardPayment,arguments: stringData)
    }
    
    public func onSettingBankBox() {
        print("***********************");
        print("ONSETTING BANKBOX CALLBACK TRIGGERED");
        print("***********************");
        MobilePosPlugin.methodChannel?.invokeMethod(RavenMPosConst.onSettingBankBox,arguments: nil)
    }
     
}
