// ignore_for_file: use_build_context_synchronously

import 'dart:io';

import 'package:flutter/material.dart';
import 'package:gap/gap.dart';
import 'package:mobile_pos/src/helpers/global_variables.dart';
import 'package:mobile_pos/src/helpers/helper_functions.dart';
import 'package:mobile_pos/src/network/api_requests.dart';
import 'package:mobile_pos/src/styles/ravenpay_textstyles.dart';
import 'package:mobile_pos/src/shared_widgets/powerby_by_raven_widget.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_amount_pad.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_button.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_close_button.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_scaffold.dart';
import 'package:mobile_pos/src/views/card_payment/connect_bluetooth.dart';

import '../../mobile_pos_platform_interface.dart';
import '../../mobile_pos_sdk.dart';

class PurchaseAmount extends StatefulWidget {
  final Function(BuildContext conext, double amount) onProceed;
  final String? title;
  const PurchaseAmount({super.key, required this.onProceed, this.title});

  @override
  State<PurchaseAmount> createState() => _PurchaseAmountState();
}

class _PurchaseAmountState extends State<PurchaseAmount> {
  String amount = "";
  bool isBusy = false;
  @override
  Widget build(BuildContext context) {
    void processPayment() async {
      final res = await MobilePosPlatform.instance
          .checkConnectivityStatus(ConnectivityType.bluetooth);
      if (res != true) {
        pluginConfig.onError.call(
            BankBoxException(code: kNibbsError, message: 'Payment failed'));
        return;
      } else {
        String? cardData;

        cardData = await MobilePosPlatform.instance.chargeCard(context,
            amount: double.parse(amount),
            accountType: '00',
            connectivityType: ConnectivityType.bluetooth);

        if (Platform.isIOS) {
          //await Future.delayed(const Duration(milliseconds: 500));
          makeButtonClickable();
          pushRoute(
              context,
              ConnectBluetooth(
                amount: double.tryParse(amount) ?? 0.0,
              ));
          return;
        }
        makeButtonClickable();
        if (cardData != null) {
          await ApiRequests.processCard(
              context, double.parse(amount), cardData);
        } else {
          pluginConfig.onError.call(
              BankBoxException(code: kNibbsError, message: 'Payment failed'));
        }
      }
    }

    return RavenPayScaffold(
      body: Column(
        children: [
          const Gap(24),
          Padding(
            padding: EdgeInsets.symmetric(horizontal: kHoriontalScreenPadding),
            child: Column(
              children: [
                const RavenPayCloseButton(
                  isArrow: true,
                  text: "",
                ),
                const Gap(24),
                Align(
                    alignment: Alignment.centerLeft,
                    child:
                        Text(widget.title ?? "Enter Amount", style: headling1)),
                const Gap(70),
                Align(
                    alignment: Alignment.center,
                    child: RichText(
                        text: TextSpan(
                            text: '₦',
                            style: headling1.copyWith(
                                fontFamily: 'Roboto',
                                fontSize: 38,
                                fontWeight: FontWeight.bold),
                            children: [
                          TextSpan(
                              text: amount.isEmpty
                                  ? "0.00"
                                  : formatAmount(double.tryParse(amount) ?? 0),
                              style: headling1.copyWith(fontSize: 38))
                        ]))),
                const Gap(24),
              ],
            ),
          ),
          const Gap(24),
          RavenPayAmountPad(
              onChange: (val) {
                amount = val;
                setState(() {});
              },
              amount: amount),
          const Gap(24),
          Padding(
            padding: EdgeInsets.symmetric(horizontal: kHoriontalScreenPadding),
            child: RavenPayButton(
              busy: isBusy,
              enabled: amount.isNotEmpty,
              buttonText: "Proceed",
              onPressed: () async {
                bool internet = await checkInternetConnection();
                if (!internet) {
                  showSnack(context,
                      'No internet connectivity, please connect to internet to proceed');
                  return;
                }

                if (internet) {
                  if (keyDetails == null) {
                    var status = await ApiRequests.fetchAppInfoBool(context);

                    if (!status) {
                      showSnack(context,
                          'No credential found, click proceed to download');
                      return;
                    }
                  }
                }

                try {
                  var val = double.tryParse(amount);
                  if (val! <= 0) {
                    return;
                  }
                  isBusy = true;
                  setState(() {});
                  processPayment();

                  //widget.onProceed(context, val);
                } catch (ex) {
                  return;
                }
              },
            ),
          ),
          const Gap(24),
          const PoweredByRaven(
            fontSize: 9,
          ),
          const Gap(24),
        ],
      ),
    );
  }

  void makeButtonClickable() {
    isBusy = false;
    setState(() {});
  }
}
