import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:gap/gap.dart';
import 'package:mobile_pos/mobile_pos_platform_interface.dart';
import 'package:mobile_pos/src/helpers/global_variables.dart';
import 'package:mobile_pos/src/helpers/helper_functions.dart';
import 'package:mobile_pos/src/helpers/logger.dart';
import 'package:mobile_pos/src/models/bluetooth_model.dart';
import 'package:mobile_pos/src/models/raven_const.dart';
import 'package:mobile_pos/src/shared_widgets/progress_dialog.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_button_loader.dart';
import 'package:mobile_pos/src/styles/ravenpay_app_colors.dart';
import 'package:mobile_pos/src/styles/ravenpay_textstyles.dart';
import 'package:mobile_pos/src/views/card_payment/insert_card.dart';
import 'package:mobile_pos/src/views/card_payment/widget/see_how_to_connect.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_button.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_card.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_close_button.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_scaffold.dart';

class ConnectBluetooth extends StatefulWidget {
  final double amount;
  const ConnectBluetooth({super.key, required this.amount});

  @override
  State<ConnectBluetooth> createState() => _ConnectBluetoothState();
}

class _ConnectBluetoothState extends State<ConnectBluetooth> {
  StreamSubscription? streamSubscription;
  List<BluetoothModel> availableDevies = [];
  bool isLoading = false;
  bool scanning = true;
  bool? status;
  @override
  void initState() {
    super.initState();
    streamSubscription = MobilePosPlatform
        .instance.channelStreamController!.stream
        .listen((event) {
      isLoading = false;
      setState(() {});
      if (event.eventType == RavenMPosEventType.searchingDevices) {
        availableDevies = event.data;
        setState(() {});
      }
      if (event.eventType == RavenMPosEventType.scanning) {
        scanning = true;
        setState(() {});
      }
      if (event.eventType == RavenMPosEventType.stopScanning) {
        scanning = false;
        setState(() {});
      }
      if (event.eventType == RavenMPosEventType.connectionErr) {
        showSnack(context, "unable to connect device... please try again");
        setState(() {});
      }
      if (event.eventType == RavenMPosEventType.settingBankBox) {
        showDialog(
            barrierDismissible: false,
            context: context,
            builder: (BuildContext context) => const ProgressDialog(
                  status: 'Setting up Bankbox...',
                ));
      }
      if (event.eventType == RavenMPosEventType.connected) {
        logData("device connected successfully...");
        setState(() {});
      }
      if (event.eventType == RavenMPosEventType.card) {
        replaceRoute(
            context,
            InsertCard(
              deviceName: availableDevies[currentIndex].title.toUpperCase(),
              amount: widget.amount,
            ));
        streamSubscription!.cancel();
      }
    });
  }

  int currentIndex = 0;
  @override
  Widget build(BuildContext context) {
    Size size = MediaQuery.of(context).size;
    return RavenPayScaffold(
        showLogo: true,
        body: Padding(
          padding: EdgeInsets.symmetric(horizontal: kHoriontalScreenPadding),
          child: Column(
            children: [
              const Gap(24),
              Row(
                children: [
                  RavenPayCloseButton(
                    isArrow: true,
                    text: "",
                    onBackPressed: () {
                      streamSubscription!.cancel();
                      streamSubscription = null;
                      Navigator.pop(context);
                    },
                  ),
                  const Spacer(),
                  const SeeHowToConnect()
                ],
              ),
              Gap(size.height * 0.08),
              const CirculBluetoothImage(),
              const Gap(16),
              AnimatedOpacity(
                opacity: scanning ? 1 : 0,
                duration: const Duration(seconds: 1),
                child: Text(
                  "Searching for device...",
                  style: subtitle.copyWith(color: pluginTheme.primaryColor),
                ),
              ),
              const Spacer(),
              Padding(
                padding: const EdgeInsets.symmetric(horizontal: 4),
                child: Row(
                  children: [
                    Expanded(
                      child: Text("Available Connections",
                          style: subtitle2.copyWith(
                              color: pluginTheme.primaryColor)),
                    ),
                    AnimatedOpacity(
                      opacity: scanning ? 0 : 1,
                      duration: const Duration(milliseconds: 500),
                      child: InkWell(
                        splashColor: Colors.transparent,
                        focusColor: Colors.transparent,
                        hoverColor: Colors.transparent,
                        highlightColor: Colors.transparent,
                        onTap: () {
                          MobilePosPlatform.instance.scanForMPosDevice();
                        },
                        child: Row(
                          children: [
                            Text("Refresh",
                                style: subtitle2.copyWith(
                                    color: pluginTheme.primaryColor)),
                            Icon(
                              Icons.refresh_rounded,
                              size: 20,
                              color: pluginTheme.primaryColor,
                            )
                          ],
                        ),
                      ),
                    )
                  ],
                ),
              ),
              const Gap(8),
              RavenPayCard(
                  padding: EdgeInsets.zero,
                  height: 180,
                  child: availableDevies.isEmpty
                      ? Center(
                          child: scanning
                              ? RavenButtonLoader(
                                  color: pluginTheme.primaryColor,
                                  size: 30,
                                )
                              : Text("No Device Found Yet",
                                  style: subtitle.copyWith(fontSize: 12)))
                      : ListView.builder(
                          itemCount: availableDevies.length,
                          itemBuilder: (context, index) {
                            return ItemView(
                                currentIndex: currentIndex,
                                name:
                                    availableDevies[index].title.toUpperCase(),
                                index: index,
                                onPressed: (val) {
                                  currentIndex = val;
                                  setState(() {});
                                });
                          })),
              const Gap(34),
              RavenPayButton(
                enabled: availableDevies.isNotEmpty,
                buttonText: "Connect to device",
                busy: isLoading,
                onPressed: () {
                  if (availableDevies.isEmpty) {
                    return;
                  }
                  processConnection();
                },
              ),
              if (pluginConfig.showLabel) ...[
                const Gap(64),
              ]
            ],
          ),
        ));
  }

  processConnection() {
    showCupertinoDialog(
        context: context,
        builder: (context) => CupertinoAlertDialog(
              content: Text("Proceed to connect with this device ?",
                  style: subtitle.copyWith(fontSize: 12, color: Colors.black)),
              title: Padding(
                padding: const EdgeInsets.only(bottom: 12),
                child: Text(availableDevies[currentIndex].title.toUpperCase(),
                    style: subtitle.copyWith(color: Colors.black)),
              ),
              actions: [
                CupertinoDialogAction(
                  onPressed: () {
                    Navigator.pop(context);
                  },
                  child: const Text("No"),
                ),
                CupertinoDialogAction(
                  child: const Text("Yes"),
                  onPressed: () {
                    isLoading = true;
                    setState(() {});
                    MobilePosPlatform.instance.connectBluetooth(
                        availableDevies[currentIndex].address);

                    Navigator.pop(context);
                  },
                )
              ],
            ));
  }
}

class CirculBluetoothImage extends StatelessWidget {
  const CirculBluetoothImage({
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    return Container(
      height: 130,
      width: 130,
      padding: const EdgeInsets.all(18),
      decoration:
          const BoxDecoration(shape: BoxShape.circle, color: Color(0xFFF2F2F2)),
      child: Container(
        padding: const EdgeInsets.all(18),
        decoration: const BoxDecoration(
            shape: BoxShape.circle, color: Color.fromARGB(255, 236, 236, 236)),
        child: Container(
            padding: const EdgeInsets.all(14),
            decoration: BoxDecoration(
                shape: BoxShape.circle, color: pluginTheme.primaryColor),
            child: Image.asset(loadAsset("bluetooth_icon.png"))),
      ),
    );
  }
}

class ItemView extends StatelessWidget {
  final String name;
  final int index;
  final int currentIndex;
  final Function(int) onPressed;
  const ItemView({
    super.key,
    required this.index,
    required this.currentIndex,
    required this.onPressed,
    required this.name,
  });

  @override
  Widget build(BuildContext context) {
    return InkWell(
      splashColor: Colors.transparent,
      highlightColor: Colors.transparent,
      onTap: () => onPressed(index),
      child: Container(
        padding: const EdgeInsets.only(bottom: 8, top: 4),
        decoration: BoxDecoration(
            border: Border(
                bottom: BorderSide(
                    color: AppColors.ravenPayGrey2.withOpacity(0.5)))),
        child: Column(
          children: [
            Row(
              children: [
                Radio(
                    activeColor: pluginTheme.primaryColor,
                    value: currentIndex,
                    groupValue: index,
                    onChanged: (val) {}),
                RavenPayText.bodyText(
                  name,
                  fontSize: 14,
                )
              ],
            ),
          ],
        ),
      ),
    );
  }
}
